syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface LogControl
/// \brief A message that allows for control of logging functions

message LogControl
{
  optional bool start            = 1;
  optional bool stop             = 2;
  optional bool paused           = 3;
  optional string base_path      = 4;
  optional string encoding       = 5;
  optional bool record_resources = 6;
}
