syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Vector2d 
/// \brief Message for a vector2 double


message Vector2d
{
  required double x = 1;
  required double y = 2;
}


