syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Header 
/// \brief General information included by many messages


import "time.proto";

message Header
{
  optional string str_id = 1;
  optional Time stamp    = 2;
  optional int32 index   = 3;
} 

