syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Response 
/// \brief Message that encapsulates a respons message with a type description


message Response
{
  required int32 id              = 1;
  required string request        = 2;
  required string response       = 3;
  optional string type           = 4;
  optional bytes serialized_data = 5;
}


